@@ -4,13 +4,17 @@ except ImportError:
     import json  # flake8: noqa
 import requests
 
-from urllib import urlencode
+try:
+    from urllib.parse import urlencode
+except ImportError:
+    from urllib import urlencode
+from decimal import Decimal
 
 from facepy.exceptions import *
 
 
 class GraphAPI(object):
-    def __init__(self, oauth_token=False, url='https://graph.facebook.com'):
+    def __init__(self, oauth_token=False, url='https://graph.facebook.com', verify_ssl_certificate=True):
         """
         Initialize GraphAPI with an OAuth access token.
 
@@ -19,6 +23,19 @@ class GraphAPI(object):
         self.oauth_token = oauth_token
         self.session = requests.session()
         self.url = url.strip('/')
+        self.verify_ssl_certificate = verify_ssl_certificate
+
+    @classmethod
+    def for_application(self, id, secret_key):
+        """
+        Initialize GraphAPI with an OAuth access token for an application.
+
+        :param id: An integer describing a Facebook application.
+        :param secret_key: A String describing the Facebook application's secret key.
+        """
+        from facepy.utils import get_application_access_token
+
+        return GraphAPI(get_application_access_token(id, secret_key))
 
     def get(self, path='', page=False, retry=3, **options):
         """
@@ -30,6 +47,9 @@ class GraphAPI(object):
         :param retry: An integer describing how many times the request may be retried.
         :param options: Graph API parameters such as 'limit', 'offset' or 'since'.
 
+        Floating-point numbers will be returned as :class:`decimal.Decimal`
+        instances.
+
         See `Facebook's Graph API documentation <http://developers.facebook.com/docs/reference/api/>`_
         for an exhaustive list of parameters.
         """
@@ -177,12 +197,23 @@ class GraphAPI(object):
         :param page: A boolean describing whether to return an iterator that iterates over each page of results.
         :param retry: An integer describing how many times the request may be retried.
         """
+        
+        if(data):
+            data = dict(
+                 (k.replace('__', ':'), v) for k, v in data.items())
         data = data or {}
 
         def load(method, url, data):
+            for key in data:
+                value = data[key]
+
+                if isinstance(value, (list, dict, set)):
+                    data[key] = json.dumps(value)
+
             try:
                 if method in ['GET', 'DELETE']:
-                    response = self.session.request(method, url, params=data, allow_redirects=True)
+                    response = self.session.request(method, url, params=data, allow_redirects=True,
+                     verify=self.verify_ssl_certificate)
 
                 if method in ['POST', 'PUT']:
                     files = {}
@@ -194,9 +225,10 @@ class GraphAPI(object):
                     for key in files:
                         data.pop(key)
 
-                    response = self.session.request(method, url, data=data, files=files)
+                    response = self.session.request(method, url, data=data, files=files, 
+                        verify=self.verify_ssl_certificate)
             except requests.RequestException as exception:
-                raise HTTPError(exception.message)
+                raise HTTPError(exception)
 
             result = self._parse(response.content)
 
@@ -220,7 +252,7 @@ class GraphAPI(object):
 
         # Convert option lists to comma-separated values.
         for key in data:
-            if isinstance(data[key], (list, set, tuple)) and all([isinstance(item, basestring) for item in data[key]]):
+            if isinstance(data[key], (list, set, tuple)) and all([isinstance(item, str) for item in data[key]]):
                 data[key] = ','.join(data[key])
 
         # Support absolute paths too
@@ -249,8 +281,11 @@ class GraphAPI(object):
 
         :param data: A string describing the Graph API's response.
         """
+        # tests seems to pass a str, while real usage bytes which should be expected
+        if type(data) == type(bytes()):
+            data = data.decode('utf-8')
         try:
-            data = json.loads(data)
+            data = json.loads(data, parse_float=Decimal)
         except ValueError:
             return data
 
